(****************************************************************************)
(*                          REMOVABLE WINDOWS                               *)
(*                                                                          *)
(*              Call INITWIN before calling  MKWIN or RMWIN.                *)
(*                                                                          *)
(*              Uses some routines from the BASIC.INC Package.              *)
(****************************************************************************)

const
   maxwin    = 30;                { Adjustable !!! }

type
   labeltype  = string[60];
   imagetype  = array[1..3840] of char;
   windimtype = record
                   x1,y1,x2,y2  : byte;
                end;
var
   win         : record
                    xh,yh : byte;
                    dim   : windimtype;
                    depth : integer;
                    stack : array[1..maxwin] of
                               record
                                  images   : ^imagetype;
                                  dim      : windimtype;
                                  x,y      : integer;
                               end;
                 end;
   crtmode     : byte      absolute $0040:$0049;
   monobuffer  : imagetype absolute $B000:$0000;
   colorbuffer : imagetype absolute $B800:$0000;

(****************************************************************************)
(*                          Initial Window System                           *)
(****************************************************************************)
procedure
   initwin;
begin
   with win.dim do begin
      x1 := 1;
      y1 := 1;
      x2 := 80;
      y2 := 24;
   end;
   win.depth := 0;
end;

(****************************************************************************)
(*                           DRAW BOX FOR WINDOW                            *)
(****************************************************************************)
procedure
   boxwin(x1,y1,x2,y2 : integer; winlabel : labeltype);
var
   x,y       : integer;
begin
   window(x1,y1,x2,y2);
   clrscr;
   window(1,1,80,25);

   { TOP }
   gotoxy(x1,y1);
   write(bstring(x2-x1+1,#220));

   { SIDES }
   for y := y1+1 to y2-1 do begin
      gotoxy(x1,y);
      write(#221);
      gotoxy(x2,y);
      write(#222);
   end;

   { BOTTOM }
   gotoxy(x1,y2);
   write(bstring(x2-x1+1,#223));

   if length(winlabel)>0 then begin
      gotoxy(x1+4,y1);
      write('  ',winlabel,'  ');
   end;

   window(x1+1,y1+1,x2-1,y2-1);
   gotoxy(1,1);
end;

(****************************************************************************)
(*                               MAKE A WINDOW                              *)
(****************************************************************************)
procedure
   mkwin(x1,y1,x2,y2 : integer; winlabel : labeltype);
begin
   normvideo;
   with win do depth := depth+1;
   if (win.depth > maxwin) or ((maxavail > 0) and (maxavail < 320)) then begin
      writeln;
      writeln(^G^G,' Windows nested too deep ');
      halt;
   end;

   new(win.stack[win.depth].images);
   if crtmode = 7 then
      win.stack[win.depth].images^ := monobuffer
   else
      win.stack[win.depth].images^ := colorbuffer;

   win.stack[win.depth].dim := win.dim;
   win.stack[win.depth].x   := wherex;
   win.stack[win.depth].y   := wherey;

   boxwin(x1,y1,x2,y2,winlabel);
   win.dim.x1 := x1+1;
   win.dim.y1 := y1+1;

   win.dim.x2 := x2-1;
   win.dim.y2 := y2-1;
end;

(****************************************************************************)
(*                       REMOVE THE PREVIOUS WINDOW                         *)
(****************************************************************************)
procedure
   rmwin;
begin
   if crtmode = 7 then
      monobuffer := win.stack[win.depth].images^
   else
      colorbuffer := win.stack[win.depth].images^;
   dispose(win.stack[win.depth].images);
   with win do begin
      dim := stack[depth].dim;
      window(dim.x1,dim.y1,dim.x2,dim.y2);
      gotoxy(stack[depth].x,stack[depth].y);
      depth := depth-1;
      if depth=0 then begin
         textcolor( FGcolor );
         textbackground( BGcolor );
      end;
   end;
end;

(****************************************************************************)
(*                           ESCAPE FROM WINDOW                             *)
(****************************************************************************)
procedure
   escape_win;
begin
   with win do begin
      xh:=wherex;
      yh:=wherey;
      window(1,1,80,25);
   end;
   textcolor( FGcolor );
   textbackground( BGcolor );
end;

(****************************************************************************)
(*                 REENTER WINDOW PREVIOUSLY ESCAPED FROM                   *)
(****************************************************************************)
procedure
   reset_win;
begin
   with win do begin
      window(dim.x1,dim.y1,dim.x2,dim.y2);
      gotoxy(xh,yh);
      if depth>0 then normvideo;
   end;
end;